{% load i18n %}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="{{ LANGUAGE_CODE }}" xml:lang="{{ LANGUAGE_CODE }}" {% if LANGUAGE_BIDI %}dir="rtl"{% endif %}><head>
<title>Notifications {% block title %}{% endblock %}</title>
{% block style %}
<style type="text/css">
.notice {
    border: solid #CCC;
    border-width :0 1px 1px 0;
    background-color: #EDF3FE;
    margin: 10px 0px 10px 0px;
    padding: 10px;
    width: 250px;
}

.first-notice {
    font-weight: bold;
}

.notice-delete {
    
}

</style>
{% endblock %}

<body>
<div class="notice">
    <a href="{{ archive_all_url }}/">{% trans "Archive All" %}</a>&nbsp;&nbsp;
    <a href="{{ delete_all_url }}/"
        onClick="if(!confirm('{% trans "Are you sure you want to delete ALL notifications?" %}')) return false">
       {% trans "Delete All" %}
    </a>

</div>
{% for notice in notices %}
<div class="notice">
    <p {% if forloop.first %}class="first-notice"{% endif %}>
       {{ notice }}
    </p>
    <p>
       {% trans "Date" %}: {{ notice.added }}
    </p>
    <div class="notice-right">
       <a href="{{ archive_url }}/{{ notice.id }}/">{% trans "Archive" %}</a>&nbsp;&nbsp;
           <a class="notice-delete" href="{{ delete_url }}/{{ notice.id }}/"
           onClick="if(!confirm('{% trans "Are you sure you want to delete this notification?" %}')) return false">
               {% trans "Delete" %}
           </a>
    </div>
</div>
{% endfor %}
</body>
</html>
